# -*- coding: utf-8 -*-

from functools import partial

from kaio import Options


opts = Options()
get = partial(opts.get, section='Debug')


class DebugMixin(object):
    """Securty base settings"""

    # https://django-debug-toolbar.readthedocs.io/en/stable/installation.html#explicit-setup
    DEBUG_TOOLBAR_PATCH_SETTINGS = False
    DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}
    INTERNAL_IPS = ['127.0.0.1']

    @property
    def DEBUG(self):
        return get('DEBUG', False)

    @property
    def TEMPLATE_DEBUG(self):
        debug = get('TEMPLATE_DEBUG', self.DEBUG)
        for template in self.TEMPLATES:
            if template['BACKEND'] == 'django.template.backends.django.DjangoTemplates':
                template['OPTIONS']['debug'] = debug

    @property
    def ENABLE_DEBUG_TOOLBAR(self):
        enabled = get('ENABLE_DEBUG_TOOLBAR', self.DEBUG)
        if enabled:
            try:
                import debug_toolbar
            except ImportError:
                return False
            else:
                self._add_debug_toolbar_to_installed_apps()
                self._add_debug_toolbar_to_middleware()

        return enabled

    @property
    def ALLOWED_HOSTS(self):
        hosts = get('ALLOWED_HOSTS')
        return [h.strip() for h in hosts.split(',') if h]

    def _add_debug_toolbar_to_installed_apps(self):
        if 'debug_toolbar' not in self.INSTALLED_APPS:
            self.INSTALLED_APPS.append('debug_toolbar')

    def _add_debug_toolbar_to_middleware(self):
        middlewares_settings = (
            'MIDDLEWARE',  # django >= 1.10
            'MIDDLEWARE_CLASSES',  # django < 1.10
        )
        for middleware_setting in middlewares_settings:
            middlewares = getattr(self, middleware_setting, None)
            if middlewares is not None:
                if 'debug_toolbar.middleware.DebugToolbarMiddleware' not in middlewares:
                    middlewares.insert(0, 'debug_toolbar.middleware.DebugToolbarMiddleware')



# -*- coding: utf-8 -*-

from functools import partial
from kaio import Options


opts = Options()
get = partial(opts.get, section='Database')


class DatabasesMixin(object):

    @staticmethod
    def get_engine(prefix):
        """
        Retrieve the database engine.
        Only change the full engine string if there is no «backends» in it.
        """
        engine = get('{}DATABASE_ENGINE'.format(prefix), 'sqlite3')
        if 'backends' in engine:
            return engine
        return 'django.db.backends.' + engine

    def get_databases(self, prefix=''):
        return {
            'default': {
                'ENGINE': self.get_engine(prefix),
                'NAME': get('{}DATABASE_NAME'.format(prefix), '{}db.sqlite'.format(prefix.lower())),
                'USER': get('{}DATABASE_USER'.format(prefix), None),
                'PASSWORD': get('{}DATABASE_PASSWORD'.format(prefix), ''),
                'HOST': get('{}DATABASE_HOST'.format(prefix), ''),
                'PORT': get('{}DATABASE_PORT'.format(prefix), ''),
                'CONN_MAX_AGE': get('{}DATABASE_CONN_MAX_AGE'.format(prefix), 0),
                'TEST': {
                    'NAME': get('{}DATABASE_NAME'.format(prefix), None),
                }
            }
        }

    def DATABASES(self):
        return self.get_databases()



# -*- coding: utf-8 -*-

from .database import DatabasesMixin
from .cache import CachesMixin
from .compress import CompressMixin
from .paths import PathsMixin
from .logs import LogsMixin
from .filerconf import FilerMixin
from .cms import CMSMixin
from .security import SecurityMixin
from .debug import DebugMixin
from .celeryconf import CeleryMixin
from .email import EmailMixin
from .whitenoise import WhiteNoiseMixin



# encoding: utf-8

from functools import partial

from kaio import Options


opts = Options()
get = partial(opts.get, section='WhiteNoise')


class WhiteNoiseMixin(object):
    """Settings for http://whitenoise.evans.io version 3"""

    @property
    def ENABLE_WHITENOISE(self):
        enabled = get('ENABLE_WHITENOISE', False)
        if enabled:
            try:
                import whitenoise
                self._add_whitenoise_to_installed_apps()
                self._add_whitenoise_to_middleware()
            except ImportError:
                return False
        return enabled

    @property
    def WHITENOISE_AUTOREFRESH(self):
        return get('WHITENOISE_AUTOREFRESH', True)

    @property
    def WHITENOISE_USE_FINDERS(self):
        return get('WHITENOISE_USE_FINDERS', True)

    def _add_whitenoise_to_installed_apps(self):
        if 'whitenoise.runserver_nostatic' not in self.INSTALLED_APPS:
            index = self.INSTALLED_APPS.index('django.contrib.staticfiles')
            self.INSTALLED_APPS.insert(index, 'whitenoise.runserver_nostatic')

    def _add_whitenoise_to_middleware(self):
        middlewares_settings = (
            'MIDDLEWARE',  # django >= 1.10
            'MIDDLEWARE_CLASSES',  # django < 1.10
        )
        for middleware_setting in middlewares_settings:
            middlewares = getattr(self, middleware_setting, None)
            if middlewares is not None:
                if 'whitenoise.middleware.WhiteNoiseMiddleware' not in middlewares:
                    try:
                        index = middlewares.index('django.middleware.security.SecurityMiddleware') + 1
                    except ValueError:
                        index = 0
                    middlewares.insert(index, 'whitenoise.middleware.WhiteNoiseMiddleware')



# -*- coding: utf-8 -*-

from kaio import Options

opts = Options()


def get(value, default):
    return opts.get(value, default, section='Security')


class SecurityMixin(object):
    """
    Security base settings
    """

    @property
    def SECRET_KEY(self):
        return get('SECRET_KEY', u'sysadmin, change the secret key!!!!')

    @property
    def ALLOWED_HOSTS(self):
        return [h.strip() for h in get('ALLOWED_HOSTS', '*').split(',')]

    @property
    def SECURE_PROXY_SSL_HEADER_NAME(self):
        return get('SECURE_PROXY_SSL_HEADER_NAME', 'HTTP_X_FORWARDED_PROTO')

    @property
    def SECURE_PROXY_SSL_HEADER_VALUE(self):
        return get('SECURE_PROXY_SSL_HEADER_VALUE', 'https')

    @property
    def SECURE_PROXY_SSL_HEADER(self):
        # required in order to have the request.is_secure() method to work properly in https environments
        # https://docs.djangoproject.com/en/1.10/ref/settings/#secure-proxy-ssl-header
        # SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
        return self.SECURE_PROXY_SSL_HEADER_NAME, self.SECURE_PROXY_SSL_HEADER_VALUE



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# bcabezas@apsl.net

try:
    from configparser import ConfigParser, NoSectionError
except ImportError:
    pass
import os
from os.path import join, pardir, abspath
import sys


DEFAULT_CONF_NAME = "app.ini"
DEFAULT_SECTION = "Base"


def singleton(cls):
    instances = {}

    def getinstance():
        if cls not in instances:
            instances[cls] = cls()
        return instances[cls]
    return getinstance


class Option(object):
    """Option Object"""

    def __init__(self, value=None, section=None, default_value=None):
        self.value = value
        self.section = section
        self.default_value = default_value

    def __repr__(self):
        msg = u'Option(value=%r, section=%r, default=%r)'
        return msg % (self.value, self.section, self.default_value)

    def get_value_or_default(self):
        if self.value is not None:
            return self.value
        return self.default_value


@singleton
class Options(object):
    """Option Parser. By now based on ini files"""

    def __init__(self):
        """Parse initial options"""
        self.config = ConfigParser()
        self.config_file = None
        self._read_config()
        self.defaults = {}
        self.options = {}
        self._parse_options()

    def _conf_paths(self):
        paths = []
        current = abspath(".")
        while current != "/":
            paths.append(join(current, DEFAULT_CONF_NAME))
            current = abspath(join(current, pardir))
        return list(reversed(paths))

    def _read_config(self):
        try:
            self.config_file = self.config.read(self._conf_paths())[0]
        except IndexError:
            self.config_file = abspath(join('.', DEFAULT_CONF_NAME))

    def _cast_value(self, value):
        """Soportados: int, bool, str"""
        try:
            value = int(value)
        except ValueError:
            if value.lower().strip() in ["true", "t", "1", "yes"]:
                value = True
            elif value.lower().strip() in ["false", "f", "no", "0"]:
                value = False
        return value

    def _parse_options(self):
        """ Parsea fichero .ini y pone las opciones en self.options """
        for section in self.config.sections():
            for key, value in self.config.items(section=section):
                value = self._cast_value(value)
                self.options[key.upper()] = Option(value, section)

    def __iter__(self):
        """Return an iterator of options"""
        return (o for o in self.options.items())

    @property
    def sections(self):
        """Get defined sections"""
        return set(o.section for k, o in self.options.items())

    def items(self, section=None):
        """Iterate items of a section in format (name, value)"""
        if section:
            return ((k, o.value) for k, o in self.options.items() if o.section == section)
        else:
            return ((k, o.value) for k, o in self.options.items())

    def keys(self, section=None):
        """Returns all configured option names (keys)"""
        return [k for k, v in self.items(section)]

    def write(self):
        """Guarda todas las opciones definidas"""
        for name, option in self.options.items():
            if sys.version_info[0] < 3:
                try:
                    value = unicode(option.value)
                except UnicodeDecodeError:
                    value = unicode(option.value, 'utf-8')
            else:
                value = str(value)

            try:
                self.config.set(section=option.section, option=name.upper(), value=value)
            except NoSectionError:
                self.config.add_section(option.section)
                self.config.set(section=option.section, option=name, value=value)
        self._write_file()

    def _write_file(self):
        import codecs
        with codecs.open(self.config_file, 'w', "utf-8") as config_file:
            self.config.write(config_file)

    def set(self, name, value, section=DEFAULT_SECTION):
        name = name.strip()
        if type(value) == str:
            if sys.version_info[0] < 3:
                value = value.decode('utf-8')
            value = value.strip()
        if name in self.options:
            self.options[name].value = value
        else:
            self.options[name] = Option(value, section)

    def get(self, name, default=None, section=DEFAULT_SECTION):
        """Returns value, and also saves the requested default
        If value exists in environ, return environ value"""
        name = name.strip()

        try:
            self.options[name].default_value = default
            if not self.options[name].section:
                self.options[name].section = section
        except KeyError:
            self.options[name] = Option(value=None, section=section, default_value=default)

        try:
            value = self._cast_value(os.environ[name].strip())
            return value
        except KeyError:
            if self.options[name].value is not None:
                return self.options[name].value
            else:
                return default
